"""
filometro.filometro
-------------------

Fornece a API pública principal do pacote.
"""

__all__ = ['Filometro']

from typing import List, Union

from dataclasses import asdict

from filometro.deolhonafila import APIDeOlhoNaFila

from filometro.dataclasses import Posto

from filometro.enums import Zone
from filometro.enums import Modality
from filometro.enums import Situation
from filometro.enums import Immunizing
from filometro.enums import District


def _posto_factory(data: dict) -> Posto:
    """Cria um objeto Posto com base no dicionário de dados fornecido."""

    return Posto(
        equipment=data['equipamento'],
        address=data['endereco'],
        district=data['distrito'],
        zone=data['crs'],
        astrazeneca=data['astrazeneca'],
        coronavac=data['coronavac'],
        coronavac_pediatrica=data['corona_ped'],
        pfizer=data['pfizer'],
        pfizer_baby=data['pfizer_baby'],
        pfizer_bivalente=data['pfizer_bivalente'],
        pfizer_pediatrica=data['pfizer_ped'],
        janssen=data['janssen'],
        influenza=data['influenza'],
        intercambialidade=data['intercambialidade'],
        situation=data['status_fila'],
        modality=data['tipo_posto'],
        last_update=data['data_hora'],
        _index_situation=data['indice_fila'],
        _id_district=data['id_distrito'],
        _id_zone=data['id_crs'],
        _id_tb_unidades=data['id_tb_unidades'],
        _id_modality=data['id_tipo_posto']
    )


def _postos_factory(data_list: List[dict]) -> List[Posto]:
    """Cria uma lista de objetos Posto com base na lista de dados fornecido."""

    return [_posto_factory(d) for d in data_list]


class Filometro():
    """Filometro é a API príncipal do pacote.

    Fornence os métodos para coletar e filtrar os dados dos postos.
    """

    def __init__(self, _api: Union[APIDeOlhoNaFila, None] = None) -> None:
        self._api = _api or APIDeOlhoNaFila()

        self._postos = self._load_postos()

    def __repr__(self) -> str:
        return f'{self.__class__.__name__}(postos={len(self._postos)})'

    def __str__(self) -> str:
        return repr(self)

    def _load_postos(self) -> List[Posto]:
        data_list = self._api.get_data()
        return _postos_factory(data_list)

    def reload(self) -> None:
        """Recarregar os dados com as informações mais recentes."""

        postos = self._load_postos()
        self._postos = postos

    def all_postos(self) -> List[Posto]:
        """Retorna os dados de todos os postos."""

        return self._postos.copy()

    def all_postos_open(self) -> List[Posto]:
        """Retorna uma lista com todos os postos abertos no momento da busca."""

        return [
            posto for posto in self._postos
            if posto.situation != Situation.NAO_FUNCIONANDO.value
        ]

    def all_postos_closed(self) -> List[Posto]:
        """Retorna uma lista com todos os postos fechados no momento da busca."""

        return self.by_situation(Situation.NAO_FUNCIONANDO)

    def by_zone(self, zone: Zone) -> List[Posto]:
        """Retorna os dados dos postos por zona selecionada."""

        return [posto for posto in self._postos if posto.zone == zone.value]

    def by_modality(self, modality: Modality) -> List[Posto]:
        """Retorna os dados dos postos por modalidade selecionada."""

        return [posto for posto in self._postos if posto.modality == modality.value]

    def by_district(self, district: District) -> List[Posto]:
        """Retorna os dados dos postos por distrito selecionado."""

        return [posto for posto in self._postos if posto.district == district.value]

    def by_situation(self, situation: Situation) -> List[Posto]:
        """Retorna os dados dos postos por situação selecionada."""

        return [posto for posto in self._postos if posto.situation == situation.value]

    def by_immunizing(self, immunizing: Immunizing) -> List[Posto]:
        """Retorna os dados dos postos por imunizante selecionado."""

        return [posto for posto in self._postos if asdict(posto).get(immunizing.value) == '1']

    def to_dict(self) -> List[dict]:
        """Retorna uma lista de dict contendo os dados de todos os postos."""

        return [asdict(posto) for posto in self.all_postos()]



"""
filometro
---------

Esse pacote fornece diversas formas de acessar, manipular e filtrar os dados
de todos os postos de saúde que são mostrados no site 'De Olho na Fila'.
"""

__all__ = [
    'Zone',
    'Modality',
    'Situation',
    'Immunizing',
    'District',
    'Filometro'
]
__version__ = '1.2.1'
__author__ = 'Matheus Felipe'

from filometro.enums import Zone
from filometro.enums import Modality
from filometro.enums import Situation
from filometro.enums import Immunizing
from filometro.enums import District

from filometro.filometro import Filometro



"""
filometro.exceptions
--------------------

Disponibiliza as exceções que podem ocorrer no pacote.
"""


class FilometroException(Exception):
    ...


class DataCollectionError(FilometroException):
    ...



import os
import sys
from shutil import rmtree
from setuptools import setup, find_packages, Command

from filometro import __version__
from filometro import __author__


here = os.path.abspath(os.path.dirname(__file__))


class PublishCommand(Command):
    """Support setup.py publish."""

    description = 'Build and publish package in Pypi.'
    user_options = []

    @staticmethod
    def print_status(msg):
        """Prints message in bold and yellow."""
        print(f'\033[1;33m{msg}\033[0m')

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.print_status('Removing previous builds…')

            rmtree(os.path.join(here, 'dist'))
            rmtree(os.path.join(here, 'build'))

        except OSError:
            pass

        self.print_status('Build Source and Wheel distribution…')
        python = sys.executable
        os.system(f'{python} setup.py sdist bdist_wheel')

        self.print_status('Uploading the package to PyPi via Twine…')
        os.system('twine upload --config-file .pypirc --repository pypi dist/*')

        sys.exit()


with open(os.path.join(here, 'README.md'), mode='r', encoding='utf-8') as f:
    long_description = '\n' + f.read()


setup(
    name='filometro',
    version=__version__,
    description='Obtenha os dados dos postos de vacinação da covid-19 em São Paulo',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT License',
    author=__author__,
    author_email='matheusfelipeog@protonmail.com',
    url='https://github.com/matheusfelipeog/filometro',
    packages=find_packages(
        exclude=('tests',)
    ),
    install_requires=[
        'requests>=2.27.1'
    ],
    zip_safe=False,
    python_requires='>=3.7',
    project_urls={
        "Bug Tracker": "https://github.com/matheusfelipeog/filometro/issues",
        "Documentation": "https://github.com/matheusfelipeog/filometro/tree/master#documentação",
        "Source Code": "https://github.com/matheusfelipeog/filometro",
    },
    keywords=[
        'filometro', 'de-olho-na-fila', 'data', 'sao-paulo',
        'covid-19', 'vacina', 'vacinasampa', 'python', 'wrapper',
        'coronavirus'
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: Portuguese (Brazilian)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    # setup.py publish support
    cmdclass={
        'publish': PublishCommand
    }
)



"""
filometro.dataclasses
---------------------

Fornece os modelos de dados utilizados pelo pacote.
"""

__all__ = ['Posto']

from dataclasses import dataclass
from dataclasses import field

from typing import List

import re


@dataclass
class Posto():
    """Modelo de dados do Posto."""

    equipment: str                               # equipamento
    address: str = field(repr=False)             # endereço
    district: str = field(repr=False)            # distrito
    zone: str = field(repr=False)                # crs (Coordinate Reference Systems)
    maps: str = field(init=False, repr=False)
    contacts: List[str] = field(init=False, repr=False, default_factory=list)
    astrazeneca: str = field(repr=False)
    coronavac: str = field(repr=False)
    coronavac_pediatrica: str = field(repr=False)
    pfizer: str = field(repr=False)
    pfizer_baby: str = field(repr=False)
    pfizer_bivalente: str = field(repr=False)
    pfizer_pediatrica: str = field(repr=False)
    janssen: str = field(repr=False)
    influenza: str = field(repr=False)
    intercambialidade: str = field(repr=False)
    situation: str = field(repr=False)           # status_fila
    modality: str = field(repr=False)            # tipo_posto
    last_update: str                             # data_hora
    _index_situation: str = field(repr=False)    # indice_fila
    _id_district: str = field(repr=False)        # id_distrito
    _id_zone: str = field(repr=False)            # id_crs
    _id_tb_unidades: str = field(repr=False)
    _id_modality: str = field(repr=False)        # id_tipo_posto

    def __post_init__(self) -> None:
        self.contacts.extend(self._extract_contacts())
        self.address = self._extract_address()
        self.maps = self._build_maps_link()

    @staticmethod
    def _remove_substring_until_the_end(string: str, substring: str) -> str:
        new_string = string
        if substring in string:
            idx_substring = string.find(substring)
            new_string = string[:idx_substring]
        return new_string

    def _extract_address(self) -> str:
        address = self.address.upper()
        address = address.replace('ENDEREÇO:', '')

        address = self._remove_substring_until_the_end(address, 'F:')
        address = self._remove_substring_until_the_end(address, 'FONE:')
        address = self._remove_substring_until_the_end(address, 'TEL:')

        address = address.strip().strip('-').strip(',')

        return address.strip()

    def _build_maps_link(self) -> str:
        address = self.address.replace(' ', '+')

        return f'https://www.google.com.br/maps/place/{address}'

    def _extract_contacts(self) -> List[str]:
        address = self.address.replace(' ', '').upper()

        concatenated_contacts = re.split(r'F:|TEL:|FONE:', address)[-1]
        contacts = re.findall(r'\d{4}-?\d{4}', concatenated_contacts)

        for idx, contact in enumerate(contacts):
            if '-' not in contact:
                contacts[idx] = f'{contact[:4]}-{contact[4:]}'

        return contacts
